@@ -1,9 +1,8 @@
 import numpy as np
+from numba import njit as jit
 
 from poliastro.core.util import circular_velocity
 
-from .._jit import jit
-
 
 @jit
 def beta(ecc_0, ecc_f, inc_0, inc_f, argp):
@@ -31,15 +30,13 @@ def beta(ecc_0, ecc_f, inc_0, inc_f, argp):
 
 @jit
 def delta_V(V_0, ecc_0, ecc_f, beta_):
-    """Compute required increment of velocity.
-    """
+    """Compute required increment of velocity."""
     return 2 * V_0 * np.abs(np.arcsin(ecc_0) - np.arcsin(ecc_f)) / (3 * np.cos(beta_))
 
 
 @jit
 def extra_quantities(k, a, ecc_0, ecc_f, inc_0, inc_f, argp, f):
-    """Extra quantities given by the model.
-    """
+    """Extra quantities given by the model."""
     beta_ = beta(ecc_0, ecc_f, inc_0, inc_f, argp)
     V_0 = circular_velocity(k, a)
     delta_V_ = delta_V(V_0, ecc_0, ecc_f, beta_)
