@@ -18,9 +18,10 @@ from czml3.properties import (
     SolidColorMaterial,
 )
 from czml3.types import IntervalValue, TimeInterval
+from erfa import gd2gce
 
 from poliastro.bodies import Earth
-from poliastro.czml.utils import ellipsoidal_to_cartesian, project_point_on_ellipsoid
+from poliastro.czml.utils import project_point_on_ellipsoid
 from poliastro.twobody.propagation import propagate
 
 PIC_SATELLITE = (
@@ -48,8 +49,7 @@ class CZMLExtractor:
     def __init__(
         self, start_epoch, end_epoch, N, attractor=None, pr_map=None, scene3D=True
     ):
-        """
-        Orbital constructor
+        """Orbital constructor
 
         Parameters
         ----------
@@ -58,16 +58,14 @@ class CZMLExtractor:
         end_epoch: ~astropy.time.core.Time
             Ending epoch
         N: int
-            Default number of sample points.
-            Unless otherwise specified, the number
-            of sampled data points will be N when calling
-            add_orbit()
+            Default number of sample points. Unless otherwise specified, the
+            number of sampled data points will be N when calling add_orbit()
         attractor: poliastro.Body
             Attractor of the orbits
         scene3D: bool
-            Determines the scene mode. If set to true, the scene
-            is set to 3D mode, otherwise it's the orthographic
-            projection.
+            Determines the scene mode. If set to true, the scene is set to 3D
+            mode, otherwise it's the orthographic projection.
+
         """
         self.packets = []  # type: List[Packet]
         self.trajectories = []  # type: List[Any]
@@ -129,7 +127,7 @@ class CZMLExtractor:
             cords = position.represent_as(CartesianRepresentation).xyz.to(u.meter).value
             cords = np.insert(cords, 0, h.value * k, axis=0)
 
-            # flatten list
+            # Flatten list
             cart_cords += list(map(lambda x: round(x[0], rf), cords.tolist()))
 
         return cart_cords
@@ -166,7 +164,7 @@ class CZMLExtractor:
             cords = position.represent_as(CartesianRepresentation).xyz.to(u.meter).value
             cords = np.insert(cords, 0, h.value * k, axis=0)
 
-            # flatten list
+            # Flatten list
             cords = list(map(lambda x: round(x[0], rf), cords.tolist()))
             t, p = cords[0], cords[1:]
             pr_p = project_point_on_ellipsoid(
@@ -184,8 +182,7 @@ class CZMLExtractor:
 
     def _init_czml_(self):
         """
-        Only called at the initialization of the extractor
-        Builds packets.
+        Only called at the initialization of the extractor Builds packets.
         """
         pckt = Preamble(
             name="document_packet",
@@ -203,6 +200,7 @@ class CZMLExtractor:
     def _change_custom_params(self, ellipsoid, pr_map, scene3D):
         """
         Change the custom properties package.
+
         Parameters
         ----------
         ellipsoid: list(int)
@@ -210,6 +208,7 @@ class CZMLExtractor:
             representing the radii in the x, y and z axis
         pr_map: str
             A URL to the projection of the defined ellipsoid (UV map)
+
         """
 
         if pr_map is None:
@@ -244,11 +243,9 @@ class CZMLExtractor:
 
         Parameters
         ----------
-        orbit: poliastro.Orbit
-            Orbit to be added
-        pos: list [~astropy.units]
-            coordinates of ground station,
-            [u v] ellipsoidal coordinates (0 elevation)
+        pos: list [~astropy.units.Quantity]
+            Coordinates of ground station,
+            list of geodetic latitude and longitude [lon, lat] (0 elevation)
         id_description: str
             Set ground station description
         label_fill_color: list (int)
@@ -273,10 +270,14 @@ class CZMLExtractor:
                 a, b = (
                     self.cust_prop[0][0],
                     self.cust_prop[0][2],
-                )  # get semi-major and semi-minor axises
+                )  # Get semi-major and semi-minor axises
             else:
-                a, b = Earth.R.to(u.m).value, Earth.R_polar.to(u.m).value
-            pos = list(map(lambda x: x.value, ellipsoidal_to_cartesian(a, b, u0, v0)))
+                a, b = Earth.R.to_value(u.m), Earth.R_polar.to_value(u.m)
+
+            f = 1 - (b / a)  # Flattenning
+
+            pos = list(gd2gce(a, f, u0.to_value(u.rad), v0.to_value(u.rad), 0))
+
         else:
             raise TypeError(
                 "Invalid coordinates. Coordinates must be of the form [u, v] where u, v are astropy units"
@@ -330,7 +331,7 @@ class CZMLExtractor:
 
         Parameters
         ----------
-        orbit: poliastro.Orbit
+        orbit: poliastro.twobody.orbit.Orbit
             Orbit to be added
         rtol: float
             Maximum relative error permitted
@@ -339,9 +340,9 @@ class CZMLExtractor:
         groundtrack_show: bool
             If set to true, the groundtrack is
             displayed.
-        groundtrack_lead_time: double
+        groundtrack_lead_time: float
             The time the animation is ahead of the real-time groundtrack
-        groundtrack_trail_time: double
+        groundtrack_trail_time: float
             The time the animation is behind the real-time groundtrack
         groundtrack_width: int
             Groundtrack width
@@ -496,14 +497,14 @@ class CZMLExtractor:
         ----------
         positions: ~astropy.coordinates.CartesianRepresentation
             Trajectory to plot.
-        epochs: ~astropy.time.core.Time
+        epochs: ~astropy.time.Time
             Epochs for positions.
         groundtrack_show: bool
             If set to true, the groundtrack is
             displayed.
-        groundtrack_lead_time: double
+        groundtrack_lead_time: float
             The time the animation is ahead of the real-time groundtrack
-        groundtrack_trail_time: double
+        groundtrack_trail_time: float
             The time the animation is behind the real-time groundtrack
         groundtrack_width: int
             Groundtrack width
@@ -546,7 +547,7 @@ class CZMLExtractor:
 
         epochs = np.fromiter(
             map(lambda epoch: (epoch - epochs[0]).to(u.second).value, epochs),
-            dtype=np.float,
+            dtype=float,
         )
 
         positions = np.around(
