@@ -16,6 +16,8 @@ from astropy.time import Time
 from astroquery.jplhorizons import Horizons
 from scipy.interpolate import interp1d
 
+from poliastro.twobody.propagation import propagate
+
 from .frames import Planes
 from .frames.util import get_frame
 from .warnings import TimeScaleWarning
@@ -55,7 +57,7 @@ def build_ephem_interpolant(body, period, t_span, rtol=1e-5):
     """
     h = (period * rtol).to(u.day).value
     t_span = (t_span[0].to(u.day).value, t_span[1].to(u.day).value + 0.01)
-    t_values = np.linspace(*t_span, int((t_span[1] - t_span[0]) / h))
+    t_values = np.linspace(*t_span, int((t_span[1] - t_span[0]) / h))  # type: ignore
     r_values = np.zeros((t_values.shape[0], 3))
 
     for i, t in enumerate(t_values):
@@ -82,7 +84,9 @@ def _interpolate_splines(epochs, reference_epochs, coordinates, *, kind="cubic")
     # Does it have a performance impact?
     interpolant_xyz = interp1d(reference_epochs.jd, coordinates.xyz.value, kind=kind)
     interpolant_d_xyz = interp1d(
-        reference_epochs.jd, coordinates.differentials["s"].d_xyz.value, kind=kind,
+        reference_epochs.jd,
+        coordinates.differentials["s"].d_xyz.value,
+        kind=kind,
     )
 
     result_xyz = interpolant_xyz(epochs.jd) * xyz_unit
@@ -115,9 +119,7 @@ def _interpolate_sinc(epochs, reference_epochs, coordinates):
 
 # Taken from https://gist.github.com/endolith/1297227
 def _sinc_interp(x, s, u):
-    """Interpolates x, sampled at "s" instants, at "u" instants.
-
-    """
+    """Interpolates x, sampled at "s" instants, at "u" instants."""
     if len(x) != len(s):
         raise ValueError("x and s must be the same length")
 
@@ -140,7 +142,7 @@ def _get_destination_frame(attractor, plane, epochs):
     if attractor is not None:
         destination_frame = get_frame(attractor, plane, epochs)
     elif plane is Planes.EARTH_ECLIPTIC:
-        destination_frame = BarycentricMeanEcliptic
+        destination_frame = BarycentricMeanEcliptic()
     else:
         destination_frame = None
 
@@ -262,7 +264,7 @@ class Ephem:
             if not given the Solar System Barycenter will be used.
         plane : ~poliastro.frames.Planes, optional
             Fundamental plane of the frame, default to Earth Equator.
-        id_type : string, optional
+        id_type : str, optional
             Use "smallbody" for Asteroids and Comets (default), and "majorbody"
             for Planets and Satellites.
 
@@ -308,6 +310,33 @@ class Ephem:
         )
         return cls(coordinates, epochs, plane)
 
+    @classmethod
+    def from_orbit(
+        cls,
+        orbit,
+        epochs,
+        plane=Planes.EARTH_EQUATOR,
+    ):
+        """Return `Ephem` from an `Orbit` at certain epochs.
+
+        Parameters
+        ----------
+        orbit: ~poliastro.twobody.orbit.Orbit
+            Orbit.
+        epochs: ~astropy.time.Time
+            Epochs to sample the orbit positions.
+        plane: ~poliastro.frames.Planes, optional
+            Fundamental plane of the frame, default to Earth Equator.
+
+        """
+        if epochs.isscalar:
+            epochs = epochs.reshape(1)
+
+        time_of_flights = epochs - orbit.epoch
+        coordinates = propagate(orbit, time_of_flights)
+
+        return cls(coordinates, epochs, plane)
+
     def sample(self, epochs=None, *, method=InterpolationMethods.SPLINES, **kwargs):
         """Returns coordinates at specified epochs.
 
