@@ -43,14 +43,15 @@ class _PlotlyOrbitPlotter(BaseOrbitPlotter):
         ----------
         coordinates : ~astropy.coordinates.CartesianRepresentation
             Trajectory to plot.
-        label : string, optional
+        label : str, optional
             Label of the trajectory.
-        color : string, optional
+        color : str, optional
             Color of the trajectory.
         trail : bool, optional
             Fade the orbit trail, default to False.
 
         """
+
         super().plot_trajectory(coordinates, label=label, color=color, trail=trail)
 
         if not self._figure._in_batch_mode:
@@ -63,9 +64,9 @@ class _PlotlyOrbitPlotter(BaseOrbitPlotter):
         ----------
         orbit : ~poliastro.twobody.orbit.Orbit
             Orbit to plot.
-        label : string, optional
+        label : str, optional
             Label of the orbit.
-        color : string, optional
+        color : str, optional
             Color of the line and the position.
         trail : bool, optional
             Fade the orbit trail, default to False.
@@ -77,7 +78,13 @@ class _PlotlyOrbitPlotter(BaseOrbitPlotter):
             return self.show()
 
     def plot_body_orbit(
-        self, body, epoch, *, label=None, color=None, trail=False,
+        self,
+        body,
+        epoch,
+        *,
+        label=None,
+        color=None,
+        trail=False,
     ):
         """Plots complete revolution of body and current position.
 
@@ -89,7 +96,7 @@ class _PlotlyOrbitPlotter(BaseOrbitPlotter):
             Epoch of current position.
         label : str, optional
             Label of the orbit, default to the name of the body.
-        color : string, optional
+        color : str, optional
             Color of the line and the position.
         trail : bool, optional
             Fade the orbit trail, default to False.
@@ -111,12 +118,13 @@ class _PlotlyOrbitPlotter(BaseOrbitPlotter):
             Epoch of the current position, none will be used if not given.
         label : str, optional
             Label of the orbit, default to the name of the body.
-        color : string, optional
+        color : str, optional
             Color of the line and the position.
         trail : bool, optional
             Fade the orbit trail, default to False.
 
         """
+
         super().plot_ephem(ephem, epoch, label=label, color=color, trail=trail)
 
         if not self._figure._in_batch_mode:
@@ -136,9 +144,7 @@ class _PlotlyOrbitPlotter(BaseOrbitPlotter):
 
 
 class OrbitPlotter3D(_PlotlyOrbitPlotter):
-    """OrbitPlotter3D class.
-
-    """
+    """OrbitPlotter3D class."""
 
     def __init__(self, figure=None, dark=False, *, num_points=150, plane=None):
         super().__init__(figure, num_points=num_points, plane=plane)
@@ -197,9 +203,9 @@ class OrbitPlotter3D(_PlotlyOrbitPlotter):
         ----------
         orbit : ~poliastro.twobody.orbit.Orbit
             Orbit to plot.
-        label : string, optional
+        label : str, optional
             Label of the orbit.
-        color : string, optional
+        color : str, optional
             Color of the line and the position.
         trail : bool, optional
             Fade the orbit trail, default to False.
@@ -212,9 +218,7 @@ class OrbitPlotter3D(_PlotlyOrbitPlotter):
 
     @u.quantity_input(elev=u.rad, azim=u.rad, distance=u.km)
     def set_view(self, elev, azim, distance=5 * u.km):
-        """Changes 3D view.
-
-        """
+        """Changes 3D view."""
         x = distance * np.cos(elev) * np.cos(azim)
         y = distance * np.cos(elev) * np.sin(azim)
         z = distance * np.sin(elev)
@@ -326,9 +330,9 @@ class OrbitPlotter2D(_PlotlyOrbitPlotter, Mixin2D):
         ----------
         coordinates : ~astropy.coordinates.CartesianRepresentation
             Trajectory to plot.
-        label : string, optional
+        label : str, optional
             Label of the trajectory.
-        color : string, optional
+        color : str, optional
             Color of the trajectory.
         trail : bool, optional
             Fade the orbit trail, default to False.
@@ -344,6 +348,30 @@ class OrbitPlotter2D(_PlotlyOrbitPlotter, Mixin2D):
             coordinates, label=label, color=color, trail=trail
         )
 
+    def plot_ephem(self, ephem, epoch=None, *, label=None, color=None, trail=False):
+        """Plots Ephem object over its sampling period.
+
+        Parameters
+        ----------
+        ephem : ~poliastro.ephem.Ephem
+            Ephemerides to plot.
+        epoch : astropy.time.Time, optional
+            Epoch of the current position, none will be used if not given.
+        label : str, optional
+            Label of the orbit, default to the name of the body.
+        color : str, optional
+            Color of the line and the position.
+        trail : bool, optional
+            Fade the orbit trail, default to False.
+
+        """
+        if self._frame is None:
+            raise ValueError(
+                "A frame must be set up first, please use "
+                "set_orbit_frame(orbit) or plot(orbit)"
+            )
+        super().plot_ephem(ephem, epoch, label=label, color=color, trail=trail)
+
     def plot(self, orbit, *, label=None, color=None, trail=False):
         """Plots state and osculating orbit in their plane.
 
@@ -351,14 +379,15 @@ class OrbitPlotter2D(_PlotlyOrbitPlotter, Mixin2D):
         ----------
         orbit : ~poliastro.twobody.orbit.Orbit
             Orbit to plot.
-        label : string, optional
+        label : str, optional
             Label of the orbit.
-        color : string, optional
+        color : str, optional
             Color of the line and the position.
         trail : bool, optional
             Fade the orbit trail, default to False.
 
         """
+
         if trail:
             raise NotImplementedError("trail not supported yet")
 
@@ -389,7 +418,7 @@ class OrbitPlotter2D(_PlotlyOrbitPlotter, Mixin2D):
             Reference plane.
         label : str, optional
             Label of the orbit, default to the name of the body.
-        color : string, optional
+        color : str, optional
             Color of the line and the position.
         trail : bool, optional
             Fade the orbit trail, default to False.
