@@ -21,6 +21,7 @@ from poliastro.frames.util import get_frame
 from poliastro.threebody.soi import laplace_radius
 from poliastro.twobody.propagation import farnocchia, propagate
 
+from ..core.elements import coe2rv_many
 from ..core.propagation.farnocchia import delta_t_from_nu as delta_t_from_nu_fast
 from ..util import find_closest_value, norm
 from ..warnings import OrbitSamplingWarning, PatchedConicsWarning, TimeScaleWarning
@@ -43,7 +44,7 @@ except ImportError:
 
 ORBIT_FORMAT = "{r_p:.0f} x {r_a:.0f} x {inc:.1f} ({frame}) orbit around {body} at epoch {epoch} ({scale})"
 # String representation for orbits around bodies without predefined
-# reference frame
+# Reference frame
 ORBIT_NO_FRAME_FORMAT = (
     "{r_p:.0f} x {r_a:.0f} x {inc:.1f} orbit around {body} at epoch {epoch} ({scale})"
 )
@@ -77,117 +78,117 @@ class Orbit:
 
     @property
     def attractor(self):
-        """Main attractor. """
+        """Main attractor."""
         return self._state.attractor
 
     @property
     def epoch(self):
-        """Epoch of the orbit. """
+        """Epoch of the orbit."""
         return self._epoch
 
     @property
     def plane(self):
-        """Fundamental plane of the frame. """
+        """Fundamental plane of the frame."""
         return self._state.plane
 
     @cached_property
     def r(self):
-        """Position vector. """
+        """Position vector."""
         return self._state.to_vectors().r
 
     @cached_property
     def v(self):
-        """Velocity vector. """
+        """Velocity vector."""
         return self._state.to_vectors().v
 
     @cached_property
     def a(self):
-        """Semimajor axis. """
+        """Semimajor axis."""
         return self._state.to_classical().a
 
     @cached_property
     def p(self):
-        """Semilatus rectum. """
+        """Semilatus rectum."""
         return self._state.to_classical().p
 
     @cached_property
     def r_p(self):
-        """Radius of pericenter. """
+        """Radius of pericenter."""
         return self.a * (1 - self.ecc)
 
     @cached_property
     def r_a(self):
-        """Radius of apocenter. """
+        """Radius of apocenter."""
         return self.a * (1 + self.ecc)
 
     @cached_property
     def ecc(self):
-        """Eccentricity. """
+        """Eccentricity."""
         return self._state.to_classical().ecc
 
     @cached_property
     def inc(self):
-        """Inclination. """
+        """Inclination."""
         return self._state.to_classical().inc
 
     @cached_property
     def raan(self):
-        """Right ascension of the ascending node. """
+        """Right ascension of the ascending node."""
         return self._state.to_classical().raan
 
     @cached_property
     def argp(self):
-        """Argument of the perigee. """
+        """Argument of the perigee."""
         return self._state.to_classical().argp
 
     @property
     def nu(self):
-        """True anomaly. """
+        """True anomaly."""
         return self._state.to_classical().nu
 
     @cached_property
     def f(self):
-        """Second modified equinoctial element. """
+        """Second modified equinoctial element."""
         return self._state.to_equinoctial().f
 
     @cached_property
     def g(self):
-        """Third modified equinoctial element. """
+        """Third modified equinoctial element."""
         return self._state.to_equinoctial().g
 
     @cached_property
     def h(self):
-        """Fourth modified equinoctial element. """
+        """Fourth modified equinoctial element."""
         return self._state.to_equinoctial().h
 
     @cached_property
     def k(self):
-        """Fifth modified equinoctial element. """
+        """Fifth modified equinoctial element."""
         return self._state.to_equinoctial().k
 
     @cached_property
     def L(self):
-        """True longitude. """
+        """True longitude."""
         return self.raan + self.argp + self.nu
 
     @cached_property
     def period(self):
-        """Period of the orbit. """
+        """Period of the orbit."""
         return self._state.period
 
     @cached_property
     def n(self):
-        """Mean motion. """
+        """Mean motion."""
         return self._state.n
 
     @cached_property
     def energy(self):
-        """Specific energy. """
+        """Specific energy."""
         return self.v.dot(self.v) / 2 - self.attractor.k / np.sqrt(self.r.dot(self.r))
 
     @cached_property
     def e_vec(self):
-        """Eccentricity vector. """
+        """Eccentricity vector."""
         r, v = self.rv()
         k = self.attractor.k
         e_vec = ((v.dot(v) - k / (norm(r))) * r - r.dot(v) * v) / k
@@ -195,25 +196,25 @@ class Orbit:
 
     @cached_property
     def h_vec(self):
-        """Specific angular momentum vector. """
+        """Specific angular momentum vector."""
         h_vec = np.cross(self.r.to(u.km).value, self.v.to(u.km / u.s)) * u.km ** 2 / u.s
         return h_vec
 
     @cached_property
     def h_mag(self):
-        """Specific angular momentum. """
+        """Specific angular momentum."""
         h_mag = norm(self.h_vec)
         return h_mag
 
     @cached_property
     def arglat(self):
-        """Argument of latitude. """
+        """Argument of latitude."""
         arglat = (self.argp + self.nu) % (360 * u.deg)
         return arglat
 
     @cached_property
     def t_p(self):
-        """Elapsed time since latest perifocal passage. """
+        """Elapsed time since latest perifocal passage."""
         t_p = (
             delta_t_from_nu_fast(
                 self.nu.to_value(u.rad),
@@ -270,7 +271,7 @@ class Orbit:
         ----------
         attractor: Body
             Main attractor
-        coord: astropy.coordinates.SkyCoord or BaseCoordinateFrame
+        coord: ~astropy.coordinates.SkyCoord or ~astropy.coordinates.BaseCoordinateFrame
             Position and velocity vectors in any reference frame. Note that coord must have
             a representation and its differential with respect to time.
         plane : ~poliastro.frames.Planes, optional
@@ -400,9 +401,7 @@ class Orbit:
 
     @classmethod
     def from_body_ephem(cls, body, epoch=None):
-        """Return osculating `Orbit` of a body at a given time.
-
-        """
+        """Return osculating `Orbit` of a body at a given time."""
         from poliastro.bodies import Earth, Moon, Sun
 
         warn(
@@ -495,7 +494,7 @@ class Orbit:
         ----------
         new_attractor: poliastro.bodies.Body
             Desired new attractor.
-        force: boolean
+        force: bool
             If `True`, changes attractor even if physically has no-sense.
 
         Returns
@@ -509,7 +508,7 @@ class Orbit:
         elif self.attractor == new_attractor.parent:  # "Sun -> Earth"
             r_soi = laplace_radius(new_attractor)
             barycentric_position = get_body_barycentric(new_attractor.name, self.epoch)
-            # transforming new_attractor's frame into frame of attractor
+            # Transforming new_attractor's frame into frame of attractor
             new_attractor_r = (
                 ICRS(barycentric_position)
                 .transform_to(self.get_frame())
@@ -580,13 +579,13 @@ class Orbit:
 
         Parameters
         ----------
-        name : string
+        name : str
             Name of the body to query for.
         epoch : ~astropy.time.Time, optional
             Epoch, default to None.
         plane : ~poliastro.frames.Planes
             Fundamental plane of the frame.
-        id_type : string, optional
+        id_type : str, optional
             Use "smallbody" for Asteroids and Comets, and "majorbody"
             for Planets and Satellites.
 
@@ -638,7 +637,7 @@ class Orbit:
 
         Parameters
         ----------
-        name: string
+        name: str
             Name of the body to make the request.
 
         Returns
@@ -657,11 +656,11 @@ class Orbit:
         obj = SBDB.query(name, full_precision=True, **kwargs)
 
         if "count" in obj:
-            # no error till now ---> more than one object has been found
-            # contains all the name of the objects
+            # No error till now ---> more than one object has been found
+            # Contains all the name of the objects
             objects_name = obj["list"]["name"]
             objects_name_in_str = (
-                ""  # used to store them in string form each in new line
+                ""  # Used to store them in string form each in new line
             )
             for i in objects_name:
                 objects_name_in_str += i + "\n"
@@ -673,15 +672,15 @@ class Orbit:
         if "object" not in obj.keys():
             raise ValueError("Object {} not found".format(name))
 
-        a = obj["orbit"]["elements"]["a"].to(u.AU) * u.AU
+        a = obj["orbit"]["elements"]["a"]
         ecc = float(obj["orbit"]["elements"]["e"]) * u.one
-        inc = obj["orbit"]["elements"]["i"].to(u.deg) * u.deg
-        raan = obj["orbit"]["elements"]["om"].to(u.deg) * u.deg
-        argp = obj["orbit"]["elements"]["w"].to(u.deg) * u.deg
+        inc = obj["orbit"]["elements"]["i"]
+        raan = obj["orbit"]["elements"]["om"]
+        argp = obj["orbit"]["elements"]["w"]
 
         # Since JPL provides Mean Anomaly (M) we need to make
         # the conversion to the true anomaly (nu)
-        M = obj["orbit"]["elements"]["ma"].to(u.rad) * u.rad
+        M = obj["orbit"]["elements"]["ma"].to(u.rad)
         # NOTE: It is unclear how this conversion should happen,
         # see https://ssd-api.jpl.nasa.gov/doc/sbdb.html
         if ecc < 1:
@@ -749,49 +748,109 @@ class Orbit:
         )
 
     @classmethod
-    @u.quantity_input(angular_velocity=u.rad / u.s, period=u.s, hill_radius=u.m)
-    def geostationary(
-        cls, attractor, angular_velocity=None, period=None, hill_radius=None
-    ):
-        """Return the geostationary orbit for the given attractor and its rotational speed.
+    def stationary(cls, attractor):
+        """Return the stationary orbit for the given attractor and its rotational speed.
 
         Parameters
         ----------
         attractor : Body
             Main attractor.
-        angular_velocity : ~astropy.units.Quantity
-            Rotational angular velocity of the attractor.
-        period : ~astropy.units.Quantity
-            Attractor's rotational period, ignored if angular_velocity is passed.
-        hill_radius : ~astropy.units.Quantity
-            Radius of Hill sphere of the attractor (optional). Hill sphere radius(in
-            contrast with Laplace's SOI) is used here to validate the stability of the
-            geostationary orbit, that is to make sure that the orbital radius required
-            for the geostationary orbit is not outside of the gravitational sphere of
-            influence of the attractor.
-            Hill SOI of parent(if exists) of the attractor is ignored if hill_radius is not provided.
+
+        Returns
+        -------
+        Orbit
+            New orbit.
+
+
         """
+        return cls.synchronous(attractor)
 
-        if angular_velocity is None and period is None:
-            raise ValueError(
-                "At least one among angular_velocity or period must be passed"
-            )
+    @classmethod
+    @u.quantity_input(
+        ecc=u.one,
+        inc=u.deg,
+        argp=u.deg,
+        arglat=u.deg,
+        raan=u.deg,
+        period_mul=u.one,
+    )
+    def synchronous(
+        cls,
+        attractor,
+        period_mul=1 * u.one,
+        ecc=0 * u.one,
+        inc=0 * u.deg,
+        argp=0 * u.deg,
+        arglat=0 * u.deg,
+        raan=0 * u.deg,
+        epoch=J2000,
+        plane=Planes.EARTH_EQUATOR,
+    ):
+        r""" Returns an orbit where the orbital period equals the rotation rate
+        of the orbited body.  The synchronous altitude for any central body can
+        directly be obtained from Kepler's Third Law by setting the orbit period
+        P\ :sub:`sync`, equal to the rotation period of the central body
+        relative to the fixed stars D\ :sup:`*`. In order to obtain this, it's
+        important to match orbital period with sidereal rotation period.
 
-        if angular_velocity is None:
-            angular_velocity = 2 * np.pi / period
+        Parameters
+        ----------
+        attractor : Body
+            Main attractor.
+        period_mul : ~astropy.units.Quantity
+            Multiplier, default to 1 to indicate that the period of the body is
+            equal to the sidereal rotational period of the body being orbited,
+            0.5 a period equal to half the average rotational period of the body
+            being orbited, indicates a semi-synchronous orbit.
+        ecc : ~astropy.units.Quantity
+            Eccentricity,default to 0 as a stationary orbit.
+        inc : ~astropy.units.Quantity
+            Inclination,default to 0 deg.
+        raan : ~astropy.units.Quantity
+            Right ascension of the ascending node,default to 0 deg.
+        argp : ~astropy.units.Quantity
+            Argument of the pericenter,default to 0 deg.
+        arglat : ~astropy.units.Quantity, optional
+            Argument of latitude, default to 0 deg.
+        epoch : ~astropy.time.Time, optional
+            Epoch, default to J2000.
+        plane : ~poliastro.frames.Planes
+            Fundamental plane of the frame.
 
-        # Find out geostationary radius using r = cube_root(GM/(angular
-        # velocity)^2)
-        with u.set_enabled_equivalencies(u.dimensionless_angles()):
-            geo_radius = np.cbrt(attractor.k / np.square(angular_velocity.to(1 / u.s)))
+        Returns
+        -------
+        Orbit
+            New orbit.
 
-        if hill_radius is not None and geo_radius > hill_radius:
-            raise ValueError(
-                "Geostationary orbit for the given parameters doesn't exist"
-            )
+        Raises
+        ------
+        ValueError
+            If the pericenter is smaller than the attractor's radius.
 
-        altitude = geo_radius - attractor.R
-        return cls.circular(attractor, altitude)
+        Notes
+        -----
+
+        Thus:
+
+        .. math::
+
+            P_{s y n c}=D^{*} \\
+
+            a_{s y n c}=\left(\mu / 4 \pi^{2}\right)^{1 / 3}\left(D^{*}\right)^{2 / 3}\\
+
+            H_{s y n c}=a_{s y n c} - R_{p l a n e t}\\
+
+        """
+        period_sync = attractor.rotational_period * period_mul
+        a_sync = (attractor.k * (period_sync / (2 * np.pi)) ** 2) ** (1 / 3)
+        nu = arglat - argp
+        r_pericenter = (1 - ecc) * a_sync
+        if r_pericenter < attractor.R:
+            raise ValueError("The orbit for the given parameters doesn't exist")
+
+        return cls.from_classical(
+            attractor, a_sync, ecc, inc, raan, argp, nu, epoch, plane
+        )
 
     @classmethod
     def heliosynchronous(
@@ -840,6 +899,7 @@ class Orbit:
             Epoch, default to J2000.
         plane : ~poliastro.frames.Planes
             Fundamental plane of the frame.
+
         """
         mean_elements = get_mean_elements(attractor)
 
@@ -906,7 +966,7 @@ class Orbit:
     def parabolic(
         cls, attractor, p, inc, raan, argp, nu, epoch=J2000, plane=Planes.EARTH_EQUATOR
     ):
-        """Return parabolic `Orbit`.
+        """Return a parabolic `Orbit`.
 
         Parameters
         ----------
@@ -947,7 +1007,7 @@ class Orbit:
         epoch=J2000,
         plane=Planes.EARTH_EQUATOR,
     ):
-        r"""Return frozen Orbit. If any of the given arguments results in an impossibility, some values will be overwritten
+        r"""Return a frozen Orbit. If any of the given arguments results in an impossibility, some values will be overwritten
 
         To achieve frozen orbit these two equations have to be set to zero.
 
@@ -1114,11 +1174,11 @@ class Orbit:
         return cartesian.represent_as(representation, differential_class)
 
     def rv(self):
-        """Position and velocity vectors. """
+        """Position and velocity vectors."""
         return self.r, self.v
 
     def classical(self):
-        """Classical orbital elements. """
+        """Classical orbital elements."""
         return (
             self.a,
             self.ecc,
@@ -1129,7 +1189,7 @@ class Orbit:
         )
 
     def pqw(self):
-        """Perifocal frame (PQW) vectors. """
+        """Perifocal frame (PQW) vectors."""
         warn(
             "Orbit.pqw is deprecated and will be removed in a future release",
             DeprecationWarning,
@@ -1307,7 +1367,7 @@ class Orbit:
             warn("anomaly outside range, clipping", OrbitSamplingWarning, stacklevel=2)
             limits = limits.clip(-nu_max, nu_max)
 
-        nu_values = np.linspace(*limits, values)
+        nu_values = np.linspace(*limits, values)  # type: ignore
         return nu_values
 
     def sample(self, values=100, *, min_anomaly=None, max_anomaly=None):
@@ -1357,16 +1417,24 @@ class Orbit:
         else:
             nu_values = self._sample_open(values, min_anomaly, max_anomaly)
 
-        # FIXME: Refactor this insanity to remove a lot of back and forth
-        # 1. Convert to classical to extract nu values (either 1 or 3 are no ops)
-        # 2. Sample values of nu -> M -> times
-        # 3. Convert to cartesian to call the farnocchia propagator harcoded here (either 1 or 3 are no ops)
-        # 4. For each time, convert to classical to retain "slow" elements
-        # 5. For each time, compute nu0 -> M0 (same value always)
-        # 6. For each time, t -> M -> nu
-        # 7. For each time, convert classical to cartesian and assemble orbit
-        time_values = time.TimeDelta(self._generate_time_values(nu_values))
-        cartesian = propagate(self, time_values)
+        n = nu_values.shape[0]
+        rr, vv = coe2rv_many(
+            np.full(n, self.attractor.k.to(u.m ** 3 / u.s ** 2).value),
+            np.full(n, self.p.to(u.m).value),
+            np.full(n, self.ecc.value),
+            np.full(n, self.inc.to(u.rad).value),
+            np.full(n, self.raan.to(u.rad).value),
+            np.full(n, self.argp.to(u.rad).value),
+            nu_values.to(u.rad).value,
+        )
+
+        # Add units
+        rr = (rr << u.m).to(u.km)
+        vv = (vv << (u.m / u.s)).to(u.km / u.s)
+
+        cartesian = CartesianRepresentation(
+            rr, differentials=CartesianDifferential(vv, xyz_axis=1), xyz_axis=1
+        )
 
         return cartesian
 
