@@ -30,14 +30,14 @@ def time_range(start, *, periods=50, spacing=None, end=None, format=None, scale=
     ----------
     periods : int, optional
         Number of periods, default to 50.
-    spacing : Time or Quantity, optional
+    spacing : ~astropy.time.Time or ~astropy.units.Quantity, optional
         Spacing between periods, optional.
-    end : Time or equivalent, optional
+    end : ~astropy.time.Time or equivalent, optional
         End date.
 
     Returns
     -------
-    Time
+    result: ~astropy.time.Time
         Array of time values.
 
     """
@@ -74,9 +74,7 @@ def find_closest_value(value, values):
 
 @u.quantity_input(start=u.rad, stop=u.rad)
 def alinspace(start, stop=None, *, num=50, endpoint=True):
-    """Return increasing, evenly spaced angular values over a specified interval.
-
-    """
+    """Return increasing, evenly spaced angular values over a specified interval."""
     if stop is None:
         stop = start + 2 * np.pi * u.rad
 
